import { None, Option, Some } from "./option";
export declare function toEither<L = unknown, R = unknown>(cb: () => R): Either<L, R>;
export type Either<L, R> = Left<L, R> | Right<L, R>;
export declare function right<L, R>(v: R): Right<L, R>;
export declare function left<R, L>(err: L): Left<L, R>;
export declare class Right<L, R> implements EitherBase<L, R> {
    readonly v: R;
    constructor(v: R);
    readonly err: undefined;
    readonly get: R;
    readonly isRight: true;
    readonly isLeft: false;
    get getErr(): L;
    readonly toOpt: Some<R>;
    map<U>(f: (v: R) => U): Either<L, U>;
    flatMap<U>(f: (v: R) => Either<L, U>): Either<L, U>;
    mapErr<U>(): Either<U, R>;
    flatMapErr<U>(): Either<U, R>;
}
export declare class Left<L, R> implements EitherBase<L, R> {
    readonly err: L;
    constructor(err: L);
    readonly v: undefined;
    readonly isRight: false;
    readonly isLeft: true;
    get get(): R;
    readonly getErr: L;
    toOpt: None<R>;
    map<U>(): Either<L, U>;
    flatMap<U>(): Either<L, U>;
    mapErr<U>(f: (v: L) => U): Either<U, R>;
    flatMapErr<U>(f: (v: L) => Either<U, R>): Either<U, R>;
}
interface EitherBase<L, R> {
    v: R | undefined;
    err: L | undefined;
    isRight: boolean;
    isLeft: boolean;
    get: R;
    getErr: L;
    toOpt: Option<R>;
    map<U>(f: (v: R) => U): Either<L, U>;
    flatMap<U>(f: (v: R) => Either<L, U>): Either<L, U>;
    mapErr<U>(f: (v: L) => U): Either<U, R>;
    flatMapErr<U>(f: (v: L) => Either<U, R>): Either<U, R>;
}
export {};
