import { Either } from "./either";
export type Option<T> = Some<T> | None<T>;
export declare function some<T>(v: T): Some<T>;
export declare function none<T>(): None<T>;
export declare function opt<T>(v: T | null | undefined): Option<T>;
export declare class Some<T> implements OptionBase<T> {
    readonly v: T;
    constructor(v: T);
    readonly isEmpty: false;
    readonly isDefined: true;
    map<U>(f: (v: T) => U): Option<U>;
    flatMap<U>(f: (v: T) => Option<U>): Option<U>;
    readonly get: T;
    getOrThrow(): T;
    getOrElse(): T;
    orElse(): Option<T>;
    fold<U>(_: () => U, right: (value: T) => U): U;
    toEither<E>(): Either<E, T>;
}
export declare class None<T> implements OptionBase<T> {
    readonly v: undefined;
    readonly isEmpty: true;
    readonly isDefined: false;
    map<U>(): Option<U>;
    flatMap<U>(): Option<U>;
    get get(): T;
    getOrThrow(err: Error): T;
    getOrElse(def: T): T;
    orElse(def: Option<T>): Option<T>;
    fold<U>(left: () => U): U;
    toEither<E>(err: E): Either<E, T>;
}
interface OptionBase<T> {
    readonly v: T | undefined;
    readonly isEmpty: boolean;
    readonly isDefined: boolean;
    map<U>(f: (v: T) => U): Option<U>;
    flatMap<U>(f: (v: T) => Option<U>): Option<U>;
    readonly get: T;
    getOrThrow(err: Error): T;
    getOrElse(def: T): T;
    orElse(def: Option<T>): Option<T>;
    fold<U>(left: () => U, right: (value: T) => U): U;
    toEither<E>(err: E): Either<E, T>;
}
export {};
