import { None, none, Option, some, Some } from "./option";

export function toEither<L = unknown, R = unknown>(cb: () => R): Either<L, R> {
  try {
    return right(cb());
  } catch (e) {
    return left(e as L);
  }
}

export type Either<L, R> = Left<L, R> | Right<L, R>;
export function right<L, R>(v: R): Right<L, R> {
  return new Right(v);
}
export function left<R, L>(err: L): Left<L, R> {
  return new Left(err);
}

export class Right<L, R> implements EitherBase<L, R> {
  constructor(readonly v: R) {}
  readonly err: undefined = undefined;
  readonly get: R = this.v;
  readonly isRight = true as const;
  readonly isLeft = false as const;
  get getErr(): L {
    throw new Error("Right has no error");
  }
  readonly toOpt: Some<R> = some(this.v);
  map<U>(f: (v: R) => U): Either<L, U> {
    return right(f(this.v));
  }
  flatMap<U>(f: (v: R) => Either<L, U>): Either<L, U> {
    return f(this.v);
  }
  mapErr<U>(): Either<U, R> {
    return right(this.v);
  }
  flatMapErr<U>(): Either<U, R> {
    return right(this.v);
  }
}

export class Left<L, R> implements EitherBase<L, R> {
  constructor(readonly err: L) {}
  readonly v: undefined = undefined;
  readonly isRight = false as const;
  readonly isLeft = true as const;
  get get(): R {
    throw new Error("Left has no value");
  }
  readonly getErr: L = this.err;
  toOpt: None<R> = none();
  map<U>(): Either<L, U> {
    return left(this.err);
  }
  flatMap<U>(): Either<L, U> {
    return left(this.err);
  }
  mapErr<U>(f: (v: L) => U): Either<U, R> {
    return left(f(this.err));
  }
  flatMapErr<U>(f: (v: L) => Either<U, R>): Either<U, R> {
    return f(this.err);
  }
}

interface EitherBase<L, R> {
  v: R | undefined;
  err: L | undefined;
  isRight: boolean;
  isLeft: boolean;
  get: R;
  getErr: L;
  toOpt: Option<R>;
  map<U>(f: (v: R) => U): Either<L, U>;
  flatMap<U>(f: (v: R) => Either<L, U>): Either<L, U>;
  mapErr<U>(f: (v: L) => U): Either<U, R>;
  flatMapErr<U>(f: (v: L) => Either<U, R>): Either<U, R>;
}
